package com.taco.swinger.text;

import java.beans.*;
import java.io.*;
import java.util.HashMap;
import java.util.Map;

import com.taco.data.MapUtilities;
import com.taco.swinger.SwingWorker;
import com.taco.swinger.text2gui.appbase.*;

/** A simple text editor. This class can be used as a base class for
 *  text-centered applications.
 */
public class TextEditor extends FileAppBase {
    public TextEditor() { }
            
    public static void main(String[] args) {
        start(new TextEditor(), args);           
    }
        
    /** Open the argument file and then notify the argument map with the
     *  "loadedText" key mapped to the text of the file.
     */
    public boolean open(File file) {
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new FileReader(file));
        } catch (IOException ioe) {
            _argMap.putNoReturn("openError", file);
            return false;
        }
        
        StringBuffer sb = new StringBuffer((int) file.length());
        
        try {
            String line = null;
            while ((line = reader.readLine()) != null) {
               sb.append(line).append('\n');
            }
        } catch (IOException ioe2) {
            _argMap.putNoReturn("openError", file);
            return false;
        }
        
        try {
            reader.close();
        } catch (IOException ioe3) {
            ; // ignore   
        }
        
        _argMap.put("loadedText", sb.toString());
        
        setModified(false);        
        return true;
    }

    public boolean save(Map options) {
        final String data = (String) options.get("data");
        final File file = (File) options.get("file");
        
        if ((file == null) || (data == null)) {
            throw new NullPointerException();
        }
        
        boolean sync = MapUtilities.getBooleanValue(options, "sync", true);
        
        if (sync) {
            try {
                save(data, file);
                setModified(false);
            } catch (IOException ioe) {
                throw new RuntimeException("Error saving", ioe);
            }
        } else {
            SwingWorker worker = new SwingWorker() {
                public Object construct() {
                    try {                    
                        save(data, file);
                    } catch (IOException ioe) {
                        return ioe;
                    }
                    return null;
                }
                
                public void finished() {
                    Exception e = (Exception) get();                   
                    if (e == null) {                    
                        _argMap.putNoReturn("status", "finishedSaving");
                        setModified(false);
                    } else {
                        Map map = new HashMap();
                        map.put("file", file);
                        map.put("exception", e);
                        _argMap.putNoReturn("saveError", map);                          
                    }
                }
            };
            worker.start();            
        }
        return true;        
    }
    
    /** Actually do the saving. This might not be called on the event dispatch
     *  thread.
     */
    public void save(String data, File file) throws IOException {
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(file));
        } catch (IOException ioe) {
            Map map = new HashMap();
            map.put("file", file);
            map.put("exception", ioe);
            _argMap.putNoReturn("saveError", map);                          
            return;
        }
        
        writer.write(data);
                
        try {
            writer.flush();
            writer.close();
        } catch (IOException ioe) {
            ; // ignore   
        }                                
    }
}
